#include "mex.h"
#include <iostream>
#include <fstream>
#include <sstream>
#include <math.h>
#include <pthread.h>
#include <unistd.h>
#include "class_handle.hpp"

/*jscont.c stuff*/
#include <sys/types.h> 
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/joystick.h>
#include <iostream>
#include <stdint.h>

/*Time taking*/
#include <ctime>

/*File handling*/
#include <time.h>
#include <string>
#include <sstream>

#define JOY_DEV "/dev/input/js0"

#define max_axis_value = 32767


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{	
    
    // Get the command string
    char cmd[128];
	if (nrhs < 1 || mxGetString(prhs[0], cmd, sizeof(cmd)))
		mexErrMsgTxt("First input should be a command string less than 64 characters long.");
        
    // New
    if (!strcmp("new", cmd)) {
        // Check parameters
        if (nlhs != 2)
            mexErrMsgTxt("Two output expected");
        // Return a handle to a new C++ instance
        int joy_fd;
        
        printf("\n\nJoystick C/S Controller. Version: TWa20150116\n\n");
        while( ( joy_fd = open( JOY_DEV , O_RDONLY)) == -1 )
        {
            if(sleep(2) != 0)
                break;
        }
        if(joy_fd == -1) {
            printf( "Couldn't open joystick\n" );
            return;
        }

        fcntl( joy_fd, F_SETFL, O_NONBLOCK );	/* use non-blocking mode */
        
        plhs[0]  = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
        int* joy_fd_ptr;
        joy_fd_ptr = (int*) mxGetPr(plhs[0]);
        *joy_fd_ptr = joy_fd;
        
        plhs[1] = convertPtr2Mat<struct js_event>(new struct js_event);
        plhs[2] = mxCreateNumericMatrix(4,1,mxINT32_CLASS,mxREAL);
        plhs[3] = mxCreateNumericMatrix(17,1,mxUINT8_CLASS,mxREAL);
        
        return;
    }
    
    // Check there is a second input, which should be the class instance handle
    if (nrhs < 2)
		mexErrMsgTxt("Second input should be a class instance handle.");
    
    // Delete
    if (!strcmp("delete", cmd)) { 
        destroyObject<struct js_event>(prhs[2]);
        return;
    }
    
    // Get the class instance pointer from the second input
    int* joy_fd_ptr;
    joy_fd_ptr = (int*) mxGetPr(prhs[1]);
    int joy_fd = *joy_fd_ptr;
    struct js_event* js = convertMat2Ptr<struct js_event>(prhs[2]);

    if (!strcmp("read_joystick_data", cmd)) {
        
        // Check parameters
        if (nlhs < 0 || nrhs < 2)
            mexErrMsgTxt("Train: Unexpected arguments.");
        
        plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
        plhs[1] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
        plhs[2] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
        
        int *type;
        int *number;
        int *value;
        
        type   = (int*) mxGetPr(plhs[0]);
        number = (int*) mxGetPr(plhs[1]);
        value  = (int*) mxGetPr(plhs[2]);    
        
        int a = read(joy_fd, js, sizeof(struct js_event));
        //while(a > 7)
        //a = read(joy_fd, js, sizeof(struct js_event));
        
        //int a = read(joy_fd, js, sizeof(struct js_event));
        //std::cout << "number: " << *number << " value "<< *value << std::endl;
        //if (a != 8 && a != -1)
            //std::cout << sizeof(struct js_event) << std::endl;
       /* while (a == 8) {
            int a = read(joy_fd, js, sizeof(struct js_event));
            std::cout << "a = 8" << std::endl;
            struct js_event* js = convertMat2Ptr<struct js_event>(prhs[2]);
            
             std::cout << "js = " << js << std::endl;
            if (a != sizeof(struct js_event))
                
                std::cout << "a = " << a << ". js_event = " << sizeof(struct js_event) << std::endl;
            
            if (read(joy_fd, js, sizeof(struct js_event)) == -1) {
                std::cout << "a = -1" << std::endl;
                break;
            
            }
            
        }*/
            
		switch (js->type & ~JS_EVENT_INIT)
        {
			case JS_EVENT_AXIS:
                *type = 1;
                *number = js->number + 1;
                *value = (int) js->value*100/32767;
				break;
			case JS_EVENT_BUTTON:
                *type = 2;
                *number = js->number + 1;
                if (js->value > 0)
                    *value = 1;
				break;
            default:
                std::cout << "Invalid joystick event" << std::endl;
		}
        return;
        

        
    }
    
    // Got here, so command not recognized
    mexErrMsgTxt("Command not recognized.");
}
