#include "mex.h"
#include "class_handle.hpp"
#include <string.h>
#include <stdio.h>

// The class that we are interfacing to
#include <VideoRayComm.h>

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{	
    // Get the command string
    char cmd[64];
	if (nrhs < 1 || mxGetString(prhs[0], cmd, sizeof(cmd)))
		mexErrMsgTxt("First input should be a command string less than 64 characters long.");
        
    // New
    if (!strcmp("new", cmd)) {
        
        // Check parameters
        if (nlhs != 2)
            mexErrMsgTxt("Two output expected");
        // Return a handle to a new C++ instance

        plhs[0] = convertPtr2Mat<VideoRayComm>(new VideoRayComm);
        plhs[1] = convertPtr2Mat<VideoRayComm::Status_t>(new VideoRayComm::Status_t);

        return;
    }
    
    // Check there is a second input, which should be the class instance handle
    if (nrhs < 2)
		mexErrMsgTxt("Second input should be a class instance handle.");
    
    // Delete
    if (!strcmp("delete", cmd)) {
        
        // Destroy the C++ object
        destroyObject<VideoRayComm>(prhs[1]);

        destroyObject<VideoRayComm::Status_t>(prhs[2]);
        
        //destroyObject<VideoRayComm::Status_t>(prhs[2]);
        // Warn if other commands were ignored
        if (nlhs != 0 || nrhs != 3)
            mexWarnMsgTxt("Delete: Unexpected arguments ignored.");
        return;
    }

    // Get the class instance pointer from the second input
    VideoRayComm *vr = convertMat2Ptr<VideoRayComm>(prhs[1]);
    VideoRayComm::Status_t *status = convertMat2Ptr<VideoRayComm::Status_t>(prhs[2]);
  
    // Call the various class methods
    // Train 
    
    if (!strcmp("read_IMU_data", cmd)) {
        
        // Check parameters
        if (nlhs < 0 || nrhs < 2)
            mexErrMsgTxt("Train: Unexpected arguments.");
        *status = vr->send_nav_data_command();
        if (*status != VideoRayComm::Success)
            mexErrMsgTxt ("Exec Transfer Error!");

        /*Declare output array elements*/
        plhs[0]  = mxCreateDoubleMatrix(1, 1, mxREAL);
        plhs[1]  = mxCreateDoubleMatrix(1, 1, mxREAL);
        plhs[2]  = mxCreateDoubleMatrix(1, 1, mxREAL);
        plhs[3]  = mxCreateDoubleMatrix(1, 1, mxREAL);
        plhs[4]  = mxCreateDoubleMatrix(1, 1, mxREAL);
        plhs[5]  = mxCreateDoubleMatrix(1, 1, mxREAL);
        plhs[6]  = mxCreateDoubleMatrix(1, 1, mxREAL);
        plhs[7]  = mxCreateDoubleMatrix(1, 1, mxREAL);
        plhs[8]  = mxCreateDoubleMatrix(1, 1, mxREAL);
        plhs[9]  = mxCreateDoubleMatrix(1, 1, mxREAL);
        plhs[10] = mxCreateDoubleMatrix(1, 1, mxREAL);
        plhs[11] = mxCreateDoubleMatrix(1, 1, mxREAL);
        plhs[12] = mxCreateDoubleMatrix(1, 1, mxREAL);
        
        /*Declare variables*/ 
        double *roll;
        double *pitch;
        double *heading;
        double *acc_x;
        double *acc_y;
        double *acc_z;
        double *omega_x;
        double *omega_y;
        double *omega_z;
        double *mag_x;
        double *mag_y;
        double *mag_z;
        double *depth;
        
        /*Assign pointers*/
        roll     = mxGetPr(plhs[0]);
        pitch    = mxGetPr(plhs[1]);
        heading  = mxGetPr(plhs[2]);
        acc_x    = mxGetPr(plhs[3]);
        acc_y    = mxGetPr(plhs[4]);
        acc_z    = mxGetPr(plhs[5]);
        omega_x  = mxGetPr(plhs[6]);
        omega_y  = mxGetPr(plhs[7]);
        omega_z  = mxGetPr(plhs[8]);
        mag_x    = mxGetPr(plhs[9]);
        mag_y    = mxGetPr(plhs[10]);
        mag_z    = mxGetPr(plhs[11]);
        depth    = mxGetPr(plhs[12]);
        
        /*Point to vr functions. These and more can be found in comm.c and comm.h*/
        *roll    = vr->roll();
        *pitch   = vr->pitch();
        *heading = vr->heading();
        *acc_x   = vr->surge_accel();
        *acc_y   = vr->sway_accel();
        *acc_z   = vr->heave_accel();
        *omega_x = vr->roll_accel();
        *omega_y = vr->pitch_accel();
        *omega_z = vr->yaw_accel();
        *mag_x   = vr->mag_x();
        *mag_y   = vr->mag_y();
        *mag_z   = vr->mag_z();
        *depth   = vr->depth();

        return;
    }
    if (!strcmp("read_depth_data", cmd)) {
        
        // Check parameters
        if (nlhs < 0 || nrhs < 2)
            mexErrMsgTxt("Train: Unexpected arguments.");
        *status = vr->send_nav_data_command();
        if (*status != VideoRayComm::Success)
            mexErrMsgTxt ("Exec Transfer Error!");

        /*Declare output array elements*/
        plhs[0]  = mxCreateDoubleMatrix(1, 1, mxREAL);

        double *depth;
        
        /*Assign pointers*/
        depth    = mxGetPr(plhs[0]);
        
        /*Point to vr functions. These and more can be found in comm.c and comm.h*/
        *depth   = vr->depth();

        return;
    }
   
    
 
    
     /* Declare variables. */ 
    /*int surge, yaw, heave, lateral;
	int surge1, surge2;
     */
    
    double *pr;
    const mxArray *cell_element_ptr;
    int tmp0, tmp1, tmp2; //, tmp3;
    
    
    if (!strcmp("setThruster", cmd)) {
    
       pr = (double*) mxGetPr(prhs[3]);
       
       tmp0 = (int) *pr;
       tmp1 = (int) *(pr + 1);
       tmp2 = (int) *(pr + 2);
       //tmp3 = (int) *(pr + 3);
       
       int surge1  = tmp0;
       int surge2  = tmp1;
       int heave   = tmp2;
       //int lateral = tmp3;

       *status = vr->set_port_thruster(surge1); 
       *status = vr->set_starboard_thruster(surge2);
       *status = vr->set_vertical_thruster(heave);
        //lateral
        
       *status = vr->send_control_command();	
       *status = vr->send_nav_data_command();
       return;
    }
    
    if (!strcmp("setLight", cmd)) {
        
        double *current_light_ptr, *change_light_ptr;
        current_light_ptr = (double*) mxGetPr(prhs[3]);
        change_light_ptr  = (double*) mxGetPr(prhs[4]);
        
        double changeLight, currentLight;
        currentLight = (double) *current_light_ptr;
        changeLight  = (double) *change_light_ptr;
        
        int newLight = 0;
        
        //mexPrintf("currentLight = %f\n", currentLight);
        
        if( (changeLight>0) && currentLight<100 )
			newLight = (int) (currentLight + changeLight);
		
		if( (changeLight<0) && currentLight > 1 ) 
			newLight = (int) (currentLight + changeLight);
        
        
        
        return;
    }
    
    
    // Got here, so command not recognized
    mexErrMsgTxt("Command not recognized.");
}
