
x_d = NaN;
y_d = NaN;
z_d = NaN;
psi_d = NaN;
e_psi = NaN;

N1 = 0;
N2 = 0;
N3 = 0;

%Thruster input vector
u  = zeros(4,1); 

%Generate Waypoints
WP = WPgenerator(); 
k  = 1; 
%CreatePath(WP);

%Time parameters
time = 0;
loop.Timer = 0; 
dataTimer = tic;

%Storing data for plotting
dataMatrix = NaN(10000,11);
velMatrix  = NaN(10000,4);
inputData = zeros(10000,4);
forceData = zeros(10000,5);
j = 0; %Counter

freq.DepthCTRL = 0.01; %Data storage, read observer, etc. 
freq.saveData = 0.05;

[s_rol, s_pit, s_psi, a_x, a_y, a_z, a_p, a_q, a_r, mx, my, mz, d_init] = vr.read_IMU_data();
[d] = vr.read_depth_data();
depth = (d - d_init)/9.8;

figure(99);
clf;
%Got so far, entering the control loop
disp('System status: ON');
loop.Timer = tic;
while(1)
    
    %Handle all joystick events
    handle_joystick; 
    
    %Exit program whenever START-button is pressed.
    if (js.getButton(START) == 1)
        vr.setThruster([0; 0; 0; 0]);
        break;
    end
    
%Depth controller
    if (js.getButton(TRIANGLE) == 1 && time - mode.Timer > 1)
        mode.Timer = time; %Prevents "stored" js commands to trigger consecutively
        mode.Depth = changeMode(mode.Depth); %Change mode       
        if mode.Depth
            z_d = 0.3;
        else
            z_d = NaN;
        end
    elseif mode.Depth 
        handle_controller_depth;
    end 
    
%Heading Controller
    if (js.getButton(CROSS) == 1 && time - mode.Timer > 1)
        mode.Timer = time; %Prevents "stored" js commands to trigger consecutively
        mode.Heading = changeMode(mode.Heading);
    elseif mode.Heading 
        [x, y, z, psi] = qualisysGetData(sub);
        handle_controller_heading;
    end
    
%Waypoint guidance
    if(js.getButton(SQUARE) == 1 && time - mode.Timer > 1)
        mode.Timer = time;
        mode.WP = changeMode(mode.WP);
        done = 0;
    elseif mode.WP
        [x, y, z, psi] = qualisysGetData(sub);
        handle_waypoints; %Select WP, check radius of acceptance, etc.
        handle_controller_heading; %These makes up the guidance system
        handle_controller_depth;
        if done
            'Path has been followed successfully'
            break;
            
        %Thrust controller
        elseif abs(rad2deg(e_psi)) < 6 && ~isnan(psi) && abs(depth-z_d) < 0.1 
            if e_WP > 0.3
                u = u + [25; 25; 0; 0];
            else
                u = u + [12; 12; 0; 0];
            end
        end
    end
    
    if (js.getButton(CIRCLE) == 1)
        u = u + [30; -40; 0; 0];
    elseif (js.getButton(SQUARE) == 1)
        u = u + [-40; 30; 0; 0];
    elseif (js.getButton(L1) == 1)
        u = u + [20; 20; 0; 0];
    end
    
    vr.setThruster(u);  
    
    time = time + toc(loop.Timer);
    
    loop.Timer = tic;
 
    
%Read from sensor and Qualisys and store the data

    %Read sensor data
    if toc(dataTimer) > freq.saveData/2
        get_depth; %Updates the current depth variable
    end
        
    
    %Add data to matrix for plotting   
    if toc(dataTimer) > freq.saveData %10Hz, 15 Hz = 0.0667
        j = j + 1;
        
        [x, y, z, psi] = qualisysGetData(sub);
        
%         calculateVelocity;
%         velMatrix(j+1,:)  = [time, u, v, z];
        
        if ~isnan(x)
            realTimePlot;
        end
    
                            %1     2    3  4    5  6    7  8      9      10   11 
        dataMatrix(j+1,:) = [time, x_d, x, y_d, y, z_d, z, psi_d, psi, depth, j];     
        inputData(j+1,:)  = [time, u(1), u(2), u(3)];    
        forceData(j+1,:)  = [time, N1, N2, N3, j];
        dataTimer = tic;
    end

    %If attempting to go below certain limits, e.g., if the thrusters are
    %frozen in a state where they take the vessel down too quickly, the 
    %program will exit to prevent collision with the floor
    if(depth>0.6)
        a = tic;
        while (toc(a) < 0.5)
            vr.setThruster([0; 0; 100; 0]);
            disp('Exceeded depth limit');
        end
        break;
    end

end

fprintf('\nDone. Plotting...\n');

i = 1;
for i = 1:j
   if ~isnan(dataMatrix(i,3))
       break;
   end
end

%Cut off uneccessary stored zeros/NaNs
dataMatrix = dataMatrix(2:j,:); %replace "2" with "i"
inputData  = inputData(2:j,:);
forceData  = forceData(2:j,:);

plotData(dataMatrix, inputData);

figure(5);
% save('dataMatrix.mat', 'dataMatrix');

%saveData();

% folderString = savePlots;
% 
% saveData(folderString, dataMatrix);

%Delete objects
js.delete();
vr.delete();
clear vr;
clear js;
