function [N1, N2, PID_out] = audoHeading(e, PID)

dt = 0.05;

% %For path-following
Kp = 90; 
Ki = 8; 
Kd = 30;

if abs(rad2deg(e)) < 15
    int = PID.integral + (e*dt); %Forward Euler %error*dt*X
else
    int = 0;
    PID.integral = 0;
end

e_dot = (e - PID.preError)/dt;

P = Kp*e;
D = max(min(Kd*e_dot, 2), -2); %saturate
I = Ki*int;

if isnan(e)
    tau_N = 0;
else
    tau_N = P + I + D;
end

% Saturate thrusters
if abs(rad2deg(e)) > 25
    N = max(min(tau_N,40),-40);
else
    N = max(min(tau_N,30),-30);
end

%fprintf('e= %.3f \t PID.integral= %.3f \t int= %.3f | preE= %.3f \t e_dot= %.3f | N= %.3f \n', e, PID.integral, int, PID.preError, e_dot, N);

fprintf('_Heading_ P: %.2f \t D: %.2f \t I: %.2f \t Tot: %.0f \t err: %.0f \n', P, D, I, N, rad2deg(e));

%Allocate thrust
N1 = -N/2;
N2 =  N/2;

%Update struct with previous error and integral
PID_out.preError = e;
PID_out.integral = int;

end



