function a = plotData(dataMatrix, inputData)

close all;

T_end   = max(dataMatrix(:,1));
T_start = min(dataMatrix(:,1));

set(0,'DefaultFigureWindowStyle','docked') 

dataMatrix(:, 9) = deg2pipi(rad2deg(dataMatrix(:,9)));
dataMatrix(:, 8) = deg2pipi(rad2deg(dataMatrix(:,8)));

% Depth plot
for a = 1:1
figure(1);
clf;
subplot(4,1,[1 2])
grid on
hold on
plot(dataMatrix(:,1), -dataMatrix(:,10), 'LineWidth',1.5, 'Color', 'g');
plot(dataMatrix(:,1), -dataMatrix(:,6) , 'LineWidth',1.5, 'Color', 'r');
legend('Depth', 'Desired depth', 'Location', 'Best');
line([0, T_end],[-1.25, -1.25],'LineWidth',2,'Color','k')
ylabel('Depth - [m]','Interpreter','latex')
axis([0 T_end -1.25 0.1]);

%Error depth plot
subplot(4,1,[3 4])
grid on
hold on
plot(dataMatrix(:,1), dataMatrix(:,10)-dataMatrix(:,6))
line([0, T_end],[0, 0],'LineWidth',1,'Color','k')
xlabel('Time - [s]','Interpreter','latex')
ylabel('Error depth [m]','Interpreter','latex')
axis([0 T_end -inf inf])
legend('Error')

end

%% Input plot
for a = 1:1
figure(2);
clf;

subplot(3, 1, 1)
grid on
hold on
ylabel('Input % Surge 1')
axis([0 T_end -100 100])
plot(inputData(:,1), inputData(:,2));

subplot(3, 1, 2)
grid on
hold on
ylabel('Input % Surge 2')
axis([0 T_end -100 100])
plot(inputData(:,1), inputData(:,3));

subplot(3, 1, 3)
grid on
hold on
ylabel('Input % Heave')
axis([0 T_end -100 100])
plot(inputData(:,1), inputData(:,4));


end

% 
%% Heading plot
for a = 1:1
figure(3);
clf;
subplot(4,1,[1 2])
grid on
hold on
ylabel('Heading - [deg]','Interpreter','latex')
axis([0 T_end -180 180])
plot(dataMatrix(:,1), dataMatrix(:,9))
plot(dataMatrix(:,1), dataMatrix(:,8), 'r')

legend('\psi', '\psi_d', 'Location', 'Best')

%Error heading plot
subplot(4,1,[3 4]) 
grid on
hold on
xlabel('Time - [s]','Interpreter','latex')
ylabel('Heading error - [deg]','Interpreter','latex')
axis([0 T_end -10 10])
plot(dataMatrix(:,1), deg2pipi(dataMatrix(:,8)-dataMatrix(:,9)))
legend('\psi error', 'Location', 'Best')
end

%% Velocity plot
for a = 1:1
% figure(4);
% clf;
% subplot(3,1,1)
% grid on
% hold on
% plot(sensorData(:,1), sensorData(:,5))
% legend('u')
% 
% subplot(3,1,2)
% grid on
% hold on
% plot(sensorData(:,1), sensorData(:,6))
% legend('v')
% 
% subplot(3,1,3)
% grid on
% hold on
% plot(sensorData(:,1), sensorData(:,7))
% legend('w')
end


%% XY position plot 
for a = 1:1
figure(5);
clf;

%XY
% subplot(2,2,[1 2])
grid on
hold on
xlabel('X-position [m]','Interpreter','latex')
ylabel('Y-position [m]','Interpreter','latex')
% axis([-inf 2.2 -1.5 1.1])


%plot(posMat(:,3),posMat(:,5), 'LineWidth',1.5, 'Marker', 'o')
plot(dataMatrix(:,3),dataMatrix(:,5), 'LineWidth',1.5)
plot(dataMatrix(:,2),dataMatrix(:,4), 'LineWidth',1, 'Marker', 'o')
legend('ROV Position', 'Path and waypoints');

% legend('[X_d Y_d]', '[X Y]', 'Location', 'Best')

camroll(270);
% camroll(90);
% camroll(180);

%X error time
% subplot(2,2,3)
% grid on
% hold on
% xlabel('Time [s]','Interpreter','latex')
% ylabel('X-position [m]','Interpreter','latex')
% axis([0 T_end -inf inf])
% 
% plot(dataMatrix(:,1), dataMatrix(:,2) - dataMatrix(:,3))
% legend('X error', 'Location', 'Best')
% 
% %Y error time
% subplot(2,2,4)
% grid on
% hold on
% xlabel('Time [s]','Interpreter','latex')
% ylabel('Y-position [m]','Interpreter','latex')
% axis([0 T_end -inf inf])
% 
% plot(dataMatrix(:,1), dataMatrix(:,4) - dataMatrix(:,5))
% legend('Y error', 'Location', 'Best')

%                             %1     2    3  4    5  6    7  8      9      10 
%         dataMatrix(j+1,:) = [time, x_d, x, y_d, y, z_d, z, psi_d, s_psi, depth];
end

%% XY and Heading
for a = 1:1
figure(6);
clf;

% Position
subplot(2,2,[1 2])
grid on
hold on
xlabel('X-position [m]','Interpreter','latex')
ylabel('Y-position [m]','Interpreter','latex')

plot(dataMatrix(:,3),dataMatrix(:,5), 'LineWidth',1.5)
plot(dataMatrix(:,2),dataMatrix(:,4), 'LineWidth',1, 'Marker', 'o')
legend('ROV Position', 'Path and waypoints');
camroll(270);

% eta_0 = [6.2 -.55];
% waypoints = [
%     eta_0;
%     6 -1;
%     6 0.5;
%     5.5 0.5;
%     5.5 -1;
%     5 -1;
%     5 0.5;
%     4.5 0.5;
%     4.5 -1];
% k = length(waypoints);
% for jj = 1:k
%      if jj == 1
%         txt = cellstr('Start');
%      else
%         txt = cellstr(strcat('WP ', num2str(jj-1)));
%      end
%      text(waypoints(jj,1)+0.07,waypoints(jj,2)+0.08,txt)
%      %plot(WP(j,2), WP(j,3), 'Color', 'b', 'Marker', 'o');
%  end

% Heading
subplot(2,2,[3 4])
grid on
hold on
xlabel('Time [s]','Interpreter','latex')
ylabel('Heading - [deg]','Interpreter','latex')
axis([-inf inf -180 180]);

plot(dataMatrix(:,1), dataMatrix(:,9))
plot(dataMatrix(:,1), deg2pipi(dataMatrix(:,8)))
legend('\psi', '\psi_d', 'Location', 'Best')

end


%% Case plot 1: Straight line following with constant depth
% for a = 1:1
% figure(20)
% clf;
% 
% %X Y Z and heading plot (3D plot), gjerne med animasjon?, med noen utvalgte
% %punkter, f eks. n=20 klosser(ROV) oppå linja
% n = 10;
% N = ceil(index_max/n); %Kloss hver N'te index;
% 
% xlabel('X-position [m]','Interpreter','latex')
% ylabel('Y-position [m]','Interpreter','latex')
% zlabel('Z-position [m]','Interpreter','latex')
% axis([-8 8 -8 8 -1.2 0.2])
% 
% Azimuth = 50;
% Elevation = 20;
% view(Azimuth, Elevation);
% hold on
% grid on
% 
% %init                            X          Y           Z                              PSI
% bentdrawCuboid([2;1.2;0.1], [posMat(1,4);posMat(1,6);posMat(1,8)], [0; 0; posMat(1,10)], 'y', 0.5);
% 
% %Draw boxes with space between
% for i = 2*N:N:index_max
%     hold on
%     if (abs(posMat(i,4)-posMat(i-N,4)) > 0.2 || abs(posMat(i,6)-posMat(i-N,6)) > 0.2 || abs(posMat(i,8)-posMat(i-N,8)) > 0.2 ) %only draw if change greater than 0.5 meters
%         bentdrawCuboid([2;1.2;0.1], [posMat(i,4);posMat(i,6);posMat(i,8)], [0; 0; posMat(i,10)], 'y', 0.1);
%         
%     end
%     
% end
% 
% %Draw desired endpoint
% bentdrawCuboid([0.40;0.40;0.040], [posMat(end,3);posMat(end,5);posMat(end,7)], [0; 0; posMat(i,9)], 'r', 0.8);
% 
% %Draw desired path
% for i = 1:index_max
%     bentdrawCuboid([0.05;0.05;-0.005], [posMat(i,3);posMat(i,5);posMat(i,7)], [0; 0; posMat(i,9)], 'k', 0.1);
% end
% 
% 
% legend('Videoray position', 'Desired endpoint', 'Desired path', 'Location', 'Best')
% 
% x = line([-8, 8],  [ 0,  0],  [ 0,   0], 'color','b');
% y = line([ 0  0],  [-8,  8],  [ 0,   0], 'color','b');
% z = line([ 0  0],  [ 0,  0],  [-1.2, 0.4], 'color','b');
% 
% end

%%
% set(gcf,'color','w'); makes background white instead of gray
eta_0 = [6.2 -.55];
waypoints = [
    eta_0;
    6 -1;
    6 0.5;
    5.5 0.5;
    5.5 -1;
    5 -1;
    5 0.5;
    4.5 0.5;
    4.5 -1];
k = length(waypoints);
for jj = 1:k
     if jj == 1
        txt = cellstr('Start');
     else
        txt = cellstr(strcat('WP ', num2str(jj-1)));
     end
     text(waypoints(jj,1)+0.07,waypoints(jj,2)+0.08,txt)
     %plot(WP(j,2), WP(j,3), 'Color', 'b', 'Marker', 'o');
 end
